import os
import re

# directories
CURR_DIR = os.path.dirname(os.path.realpath(__file__))
BASE_DIR = os.path.dirname(CURR_DIR)
PDFS_DIR = os.path.join(BASE_DIR, 'pdfs')
PUB_DIR = os.path.join(PDFS_DIR, '{journal_code}-Volume{volume}-Issue{issue}')
DIRS = [PDFS_DIR]

# regular expressions
## tags
TAG_STR = r'\s*%%?\s*\\%s\{(.*)\}'
TITLE_STR = r'\s*\\title\[?.*\]?\{(.*)\}'
AUTHOR_STR = r'\s*\\author\*?\[\d\]\{(.*)\}'
ABSTRACT_STR = r'\s*\\begin\{abstract\}\s*\{(.*)\}\s*\\end\{abstract\}'
TAGS = ['DOI', 'startPage',
        'received', 'revised', 'accepted',
        'journalname', 'journalyear', 'journalvolume', 'journalissue']

## dates
COVER_DATE_MONTH = {'2017': [1, 4, 7, 10],
                    '2018': [1, 4, 7, 10],
                    '2019': [1, 4, 7, 10],
                    '2020': [1, 4, 7, 10],
                    '2021': [1, 4, 7, 10],
		            '2022': [1, 4, 7, 10],
                    '2023': [1, 4, 7, 10],
                    '2024': [1, 4, 7, 10]}


JOURNAL_NAME = 'Proceedings on Privacy Enhancing Technologies'
CFP_DATES = {'2017': [{'submission': [31,  5, 2016], 'camera': [1,  9, 2016]},
                      {'submission': [31,  8, 2016], 'camera': [30, 11, 2016]},
                      {'submission': [30, 11, 2016], 'camera': [15,  3, 2017]},
                      {'submission': [28,  2, 2017], 'camera': [1,  6, 2017]},
                     ],
             '2018': [{'submission': [31,  5, 2017], 'camera': [15,  9, 2017]},
                      {'submission': [31,  8, 2017], 'camera': [15, 12, 2017]},
                      {'submission': [30, 11, 2017], 'camera': [15,  3, 2018]},
                      {'submission': [28,  2, 2018], 'camera': [15,  6, 2018]},
                     ],
             '2019': [{'submission': [31,  5, 2018], 'camera': [15,  9, 2018]},
                      {'submission': [31,  8, 2018], 'camera': [15, 12, 2018]},
                      {'submission': [30, 11, 2018], 'camera': [15,  3, 2019]},
                      {'submission': [28,  2, 2019], 'camera': [15,  6, 2019]},
					 ], 
             '2020': [{'submission': [31,  5, 2019], 'camera': [15,  9, 2019]},
                      {'submission': [31,  8, 2019], 'camera': [15, 12, 2019]},
                      {'submission': [30, 11, 2019], 'camera': [15,  3, 2020]},
                      {'submission': [29,  2, 2020], 'camera': [15,  6, 2020]},
                     ],
             '2021': [{'submission': [31,  5, 2020], 'camera': [15,  9, 2020]},
                      {'submission': [31,  8, 2020], 'camera': [15, 12, 2020]},
                      {'submission': [30, 11, 2020], 'camera': [15,  3, 2021]},
                      {'submission': [28,  2, 2021], 'camera': [15,  6, 2021]},
                     ],
             '2022': [{'submission': [31,  5, 2021], 'camera': [15,  9, 2021]},
                      {'submission': [31,  8, 2021], 'camera': [15, 12, 2021]},
                      {'submission': [30, 11, 2021], 'camera': [15,  3, 2022]},
                      {'submission': [28,  2, 2022], 'camera': [15,  6, 2022]},
                     ],
             '2023': [{'submission': [31,  5, 2022], 'camera': [15,  9, 2022]},
                      {'submission': [31,  8, 2022], 'camera': [15, 12, 2022]},
                      {'submission': [30, 11, 2022], 'camera': [15,  3, 2023]},
                      {'submission': [28,  2, 2023], 'camera': [15,  6, 2023]},
                     ],
             '2024': [{'submission': [31,  5, 2023], 'camera': [15,  9, 2023]},
                      {'submission': [31,  8, 2023], 'camera': [15, 12, 2023]},
                      {'submission': [30, 11, 2023], 'camera': [15,  3, 2024]},
                      {'submission': [28,  2, 2024], 'camera': [15,  6, 2024]},
                     ],
            }


## tex strings
DOC_RAW = r'\begin{document}'
DOC_STR = r'\\begin{document}'
DOC_REXP = re.compile(r'\\begin{document}')
# TEMPLATE_STR = r'\usepackage[big]{dgruyter_NEW}'
# TEMPLATE_REXP = re.compile(r'\usepackage\[big\]{dgruyter_NEW}')
DOCCLASS_STR = r'\documentclass[sigconf]{acmart}'
DOCCLASS_REXP = re.compile(r'\\documentclass\[sigconf.*?\]\{acmart\}')
BIBL_BACKENDS = {'backend=biber': 'biber', r'\bibliography{': 'bibtex', r'\printbibliography': 'bibtex', r'\begin{thebibliography': 'native'}

OUT_PAGES_REXP = re.compile(r'Output written on [a-zA-Z0-9_-]*\.pdf \((\d*) pages')

# commands
LATEX_CMD = 'pdflatex -shell-escape -interaction nonstopmode -halt-on-error -file-line-error {0}'

if os.name == 'nt':
	EMBED_CMD = os.path.join(CURR_DIR, "embedfont-win")
else:
	EMBED_CMD = os.path.join(CURR_DIR, "embedfont")
BIBL_CMD = '%s {0}'

MAIN_NAME = ['main', 'paper']
CDS_FILENAME = 'content_specification.doc'

## publication
ZIP_FORMAT_NAME = '{journal_code}-Volume{volume}-Issue{issue}.zip'
DOI_NAME = 'popets'
PETS_DOI = {'2017': '10.1515/%s-2017-' % DOI_NAME,
            '2018': '10.1515/%s-2018-' % DOI_NAME,
            '2019': '10.2478/%s-2019-' % DOI_NAME, 
            '2020': '10.2478/%s-2020-' % DOI_NAME,
            '2021': '10.2478/%s-2021-' % DOI_NAME,
	        '2022': '10.56553/%s-2022-' % DOI_NAME,
            '2023': '10.56553/%s-2023-' % DOI_NAME,
            '2024': '10.56553/%s-2024-' % DOI_NAME,
	    }

NUM_ISSUES = 4

## page limit
BODY_LIMIT = 13
APPENDIX_LIMIT = 999
PAGE_LIMIT = BODY_LIMIT + APPENDIX_LIMIT

# accepted papers by issue, intro from chairs counts as one paper
NUM_PAPERS = {'2017': [11, 12, 11, 20],
              '2018': [7, 7, 10, 12],
              '2019': [15, 17 , 22, 12],
              '2020': [13, 28 , 17, 25],
              '2021': [13, 20 , 23, 29],
	          '2022': [31, 27 , 35, 38],
              '2023': [36, 29 , 28, 34],
              '2024': [0, 0 , 0, 0],
}

# logger
logger = None
FMT = '%(asctime)s %(name)-12s %(levelname)-8s %(message)s'

# CSD templates
JOURN_CSD = """Journal Code: {journal_code}\n"""
ISSUE_CSD = """Volume: {volume}\nNumber: {issue_num}\nCover Date (month and year): {cover_date}\n"""
PAPER_CSD = """Filename: {filename}\nDOI: {doi}\nTitle: {title}\nAuthors: {authors}\nAbstract: {abstract}\n"""

# journal code
POPETS_ID = '10478'

# exceptions
class FontError(Exception):
    pass


class JobError(Exception):
    pass

# functions
def build_dirs():
    import shutil
    import os
    for d in DIRS:
        if os.path.isdir(d):
            shutil.rmtree(d)
        os.mkdir(d)
