import re
from datetime import datetime, timedelta

import common as cm


def replace_in_file(fpath, func, args, output=None):
    '''Replace changes by `func` in file.'''
    if output is None:
        output = fpath
    with open(fpath, 'r') as fi:
        replaced = func(*args, text=fi.read())
    with open(output, 'w') as fo:
        fo.write(replaced)


def add_cmd(cmd, text=''):
    return cm.DOC_REXP.sub(cm.DOC_STR + cmd, text)


def replace_tag(tag, value, text=''):
    '''Replace a latex tag.'''
    def replace_match(m):
        return '\n\%s{%s}\n' % (tag, value)
    def add_tag(m):
        return cm.DOC_RAW + replace_match(m)
    r = re.compile(cm.TAG_STR % tag)
    if r.search(text) is None:
        return cm.DOC_REXP.sub(add_tag, text)
    else:
        return r.sub(replace_match, text)

def get_tag_value(tag, text='', eng_only=True):
    if tag == "title":
        r = re.compile(cm.TITLE_STR)
    else:
        r = re.compile(cm.TAG_STR % tag)
    m = r.search(text)
    if m:
        return remove_non_eng_word(m.group(1)) if eng_only else m.group(1)
    else:
        return ""

def remove_non_eng_word(text):
    return re.sub("[^a-zA-Z\s\.\-:\_]+", "", text)

def set_pagenums(jobs):
    last_page = 1
    for job in jobs:
        cm.logger.info('Set page numbers for %s (total %s pages)' % (job, job.pages))

        # start page
        replace_in_file(job.main, replace_tag, ('startPage', last_page))
        last_page += job.pages

        cm.logger.debug("Last page updated to %s" % last_page)


def fill_tags(jobs):
    '''Fill tags in job's main file.'''
    for job in jobs:
        cm.logger.info('Filling tags for %s' % job)

        # DOI
        replace_in_file(job.main, replace_tag, ('acmDOI', job.doi))

        # dates
        index = int(job.issue) - 1

        # metainfo
        replace_in_file(job.main, replace_tag, ('acmYear', job.year))
        replace_in_file(job.main, replace_tag, ('copyrightyear', job.year))
        replace_in_file(job.main, replace_tag, ('acmVolume', job.year))
        replace_in_file(job.main, replace_tag, ('acmNumber', job.issue))

        # TODO: assert \subtitle is not being used (very optional because it's in the camera-ready instructions)
